/********************************************************************************************/
/*   ARDUINO RGB BITMAPS 208 x 144 px                                                       */
/********************************************************************************************/
/* Author: Raul Navarro                                                                     */
/* Web:    http://www.raulnd.com                                                            */
/* E-Mail: raulnd@gmail.com                                                                 */
/********************************************************************************************/
/*                                                                                          */
/* This program is free software: you can redistribute it and/or modify                     */
/* it under the terms of the GNU General Public License as published by                     */
/* the Free Software Foundation, either version 3 of the License, or                        */
/* (at your option) any later version.                                                      */
/*                                                                                          */
/* This program is distributed in the hope that it will be useful,                          */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of                           */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                            */
/* GNU General Public License for more details.                                             */
/*                                                                                          */
/* You should have received a copy of the GNU General Public License                        */
/* along with this program.  If not, see  < http://www.gnu.org/licenses/ >.                 */
/*                                                                                          */
/********************************************************************************************/
/*                                                                                          */
/* Render method is inspired in the example code of Javier Valcarce:                        */
/* TV Video Signal Generator with Arduino                                                   */
/* URL: http://www.javiervalcarce.eu/wiki/TV_Video_Signal_Generator_with_Arduino            */
/*                                                                                          */
/* RenderLine method is inspired in the example code of Masami Watanabe:                    */
/* Playing with Arduino (File: INT_OutDataL_160.cpp)                                        */
/* URL: http://homepage3.nifty.com/two_legs/neo/project/Arduino/proj_Arduino.htm            */
/*                                                                                          */
/* Libraries used:                                                                          */
/* TimerOne: http://code.google.com/p/arduino-timerone/                                     */
/*                                                                                          */
/********************************************************************************************/
/*                                                                                          */
/*   TESTED ON COMODORE 1084S - DIN6 CONNECTOR                                              */
/*     _______                                                                              */
/*    /5  º  1\      1: GREEN      4: RED                                                   */
/*   |4   6   2|     2: H. SYNC    5: BLUE                                                  */
/*    \___3___/      3: GND        6: V. SY                                                 */
/*                                                                                          */
/*   ARDUINO               DIN6                                                             */
/*   SYNC_PIN ------------- 2                                                               */
/*                                                                                          */
/*   R2_PIN   ---[750R]--\                                                                  */
/*   R1_PIN   ---[ 1K5]--+- 4                                                               */
/*   R0_PIN   ---[ 3K ]--/                                                                  */
/*                                                                                          */
/*   G2_PIN   ---[750R]--\                                                                  */
/*   G1_PIN   ---[ 1K5]--+- 1                                                               */
/*   G0_PIN   ---[ 3K ]--/                                                                  */
/*                                                                                          */
/*   B1_PIN   ---[750R]--+- 5                                                               */
/*   B0_PIN   ---[ 1K5]--'                                                                  */
/*                                                                                          */
/*   GND      ------------- 3                                                               */
/*                                                                                          */
/********************************************************************************************/

// Compatibilidad con Arduino 1.0 IDE
#if defined(ARDUINO) && ARDUINO >= 100
  #include "Arduino.h"
#else
  #include "WProgram.h"
#endif

// TimerOne:
#include <TimerOne.h>

#include "tools.h"
#include "frameBuffer.h"

// Sync
#define _HSYNC    0b00000001
#define _VSYNC    0b00000010
#define _SYNC     0b00000000

// Colors          RRRGGGBB
#define _BLACK   0b00000000
#define _BLUE    0b00000011
#define _GREEN   0b00011100
#define _CYAN    0b00011111
#define _RED     0b11100000
#define _MAGENTA 0b11100011
#define _YELLOW  0b11111100
#define _WHITE   0b11111111

byte _color[8] PROGMEM = { _BLACK, _BLUE, _GREEN, _CYAN, _RED, _MAGENTA, _WHITE };

// Total PAL lines
#define DISPLAY_LINES 312

// Arduino pins
#define H_PIN   53
#define V_PIN   52

#define R2_PIN   30
#define R1_PIN   31
#define R0_PIN   32
#define G2_PIN   33
#define G1_PIN   34
#define G0_PIN   35
#define B1_PIN   36
#define B0_PIN   37
  
// Delay: 1 cycle
#define NOP __asm __volatile ("nop")

// Delay: decimal us
#define DELAY_07  __asm __volatile ("nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t") // 11.2 - 1 ~ 10
#define DELAY_03  __asm __volatile ("nop\n\t""nop\n\t""nop\n\t""nop\n\t") // 4.8 - 1 ~ 4
#define DELAY_035 __asm __volatile ("nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t") // 5.6 - 1 ~ 5
#define DELAY_065 __asm __volatile ("nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t") // 10.4 - 1 ~ 9

// Sync signals
inline void BoardSync()
{
  SYNC = _SYNC; // 27.3
  delayMicroseconds(27); DELAY_03;
  SYNC = _VSYNC; // 4.7
  delayMicroseconds(4); DELAY_07;
  // 512 cliclos
}

inline void ShortSync()
{
  SYNC = _SYNC; // 2.35
  delayMicroseconds(2); DELAY_035;
  SYNC = _VSYNC; // 29.65
  delayMicroseconds(29); DELAY_065;
  // 512 cliclos
}

inline void HSync()
{
  SYNC = _SYNC; // 4.7
  delayMicroseconds(4); DELAY_07;
  SYNC = _HSYNC; // 5.7
  delayMicroseconds(5); DELAY_07;
  // 166 cliclos
}

inline void FrontPorch()
{
  SYNC = _HSYNC; // 1.65
  delayMicroseconds(1); DELAY_065;
  // 26 cliclos
}

unsigned int line;
byte nLine;

// Render a frame
// Pseudo-progressive PAL: 312 lines
void Render()
{

  for (line=0;line<DISPLAY_LINES; line++)
  {    

    switch(line)
    {
      
      case   0 ...   1:
        BoardSync();
        BoardSync();
        break;
 
      case 2:
        BoardSync();
        ShortSync();
        break;
 
      case   3 ...   4:
      case 309 ... 311:
        ShortSync();
        ShortSync();
        break;

      case   5 ...  22:
        HSync();
        delayMicroseconds(52);
        FrontPorch();
        break;

      default:
        nLine = (line-24)>>1;
        HSync();
        RenderLine(frameBuffer[nLine]);
        FrontPorch();
        break;
        
      }
    
    }
 
}


/**************************************************/


void setup()
{

  pinMode (H_PIN, OUTPUT);
  pinMode (V_PIN, OUTPUT);
  pinMode (R2_PIN, OUTPUT);
  pinMode (R1_PIN, OUTPUT);
  pinMode (R0_PIN, OUTPUT);
  pinMode (G2_PIN, OUTPUT);
  pinMode (G1_PIN, OUTPUT);
  pinMode (G0_PIN, OUTPUT);
  pinMode (B1_PIN, OUTPUT);
  pinMode (B0_PIN, OUTPUT);
  
  digitalWrite (H_PIN, HIGH);
  digitalWrite (V_PIN, HIGH);
  digitalWrite (R2_PIN, HIGH);
  digitalWrite (R1_PIN, HIGH);
  digitalWrite (R0_PIN, HIGH);
  digitalWrite (G2_PIN, HIGH);
  digitalWrite (G1_PIN, HIGH);
  digitalWrite (G0_PIN, HIGH);
  digitalWrite (B1_PIN, HIGH);
  digitalWrite (B0_PIN, HIGH);

  Timer1.initialize(20000);  // 20000us = 50Hz
  Timer1.attachInterrupt( Render );

}


void loop()
{
// Game update code
}

